(*
 *  COMS W4115
 *  YOLOP
 *
 *  Lisa Li
 *  Jonathan Liu
 *  Sasha McIntosh
 *
 *)

{ open Parser }

let digit      = ['0'-'9']
let int_t      = digit+
let string_r   = '\\'+ | [^ '"']
let string_t   = '"' string_r* '"'

rule token = parse
  [' ' '\t' '\r' '\n'] { token lexbuf } (* Whitespace *)
| "@#"     { comment lexbuf }           (* Comments *)
                                        (* Grouping *)
| '('      { LPAREN }
| ')'      { RPAREN }
| '{'      { LBRACE }
| '}'      { RBRACE }
| '['      { LBRACK }
| ']'      { RBRACK }
| ';'      { SEMI   }
| ','      { COMMA  }
                                        (* Binary ops *)
| '+'      { PLUS   }
| '-'      { MINUS  }
| '*'      { TIMES  }
| '/'      { DIVIDE }
| '='      { ASSIGN }
| "=="     { EQ     }
| "!="     { NEQ    }
| '<'      { LT     }
| "<="     { LEQ    }
| ">"      { GT     }
| ">="     { GEQ    }
| "&&"     { AND    }
| "||"     { OR     }
| '~'      { NOT    }
                                        (* Control Flow *)
| "if"     { IF     }
| "else"   { ELSE   }
| "for"    { FOR    }
| "while"  { WHILE  }
| "return" { RETURN }
                                        (* Primitives *)
| "int"    { INT    }
| "string" { STRING }
| "pixel"  { PIXEL  }
| "image"    { IMAGE  }
                                        (* Reserved *)
| "function"    { FUNCTION  }
| "read_in"     { READ_IN   }
| "write_out"   { WRITE_OUT }
| "get"         { GET       }
| "set"         { SET       } 
| "getc"        { GETC      }
| "setc"        { SETC      }
                                        (* Literals *)
| int_t    as lxm { ILITERAL(int_of_string lxm) }
| string_t as lxm { SLITERAL(lxm) }

| ['a'-'z' 'A'-'Z']['a'-'z' 'A'-'Z' '0'-'9' '_']* as lxm { ID(lxm) }
| eof { EOF }
| _ as char { raise (Failure("illegal character " ^ Char.escaped char)) }

and comment = parse
  "#@" { token lexbuf }
| _    { comment lexbuf }
