(*
 *  COMS W4115
 *  YOLOP
 *
 *  Lisa Li
 *  Jonathan Liu
 *  Sasha McIntosh
 *
 *)

open Printf

type action = Ast | Compile | Error

let out_name = ref "a"

(* Helper function that write "content" into a file named "name" *)
let write fName line =
    let out = open_out fName in
      fprintf out "%s\n" line;
      close_out out

(* note that this means we are passing the full filepath to the compiler *)
let getFileName path =
    let j = (String.rindex path '/') in
    String.sub path (j + 1) ((String.length path) - (j + (String.length ".yolop"
    +1)))

let usage =
    "Usage:\n\n" ^
    "    Compiling to AST Tree        : yolop -a <full filepath>\n" ^
    "    Compiling to C++ to stdin    : yolop -c <full filepath>\n"

let main () =
  let action = 
    if Array.length Sys.argv > 2 then
    try
        List.assoc Sys.argv.(1) [ ("-a", Ast);
                                  ("-c", Compile)]
      with
      _ -> Error
    else Error in
        let lexbuf = ignore(out_name := getFileName Sys.argv.(2)); 
                     Lexing.from_channel (open_in Sys.argv.(2)) in
        let program = Parser.program Scanner.token lexbuf in
        match action with
          Ast -> let listing = Ast.string_of_program program in
                   write (!out_name ^ ".ast") listing
        | Compile -> let listing = Translate.yolop_to_cpp program !out_name in
                   write (!out_name ^ ".cpp") listing;
        | Error -> print_string usage

let _ = main ()
